/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package lecturequiz.teacherclient.gamemodes;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.List;
import javax.swing.Timer;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.xml.ws.Holder;
import lecturequiz.server.service.QuestionInfo;
import lecturequiz.server.service.StatisticsEntry;
import lecturequiz.server.service.GameStatus;
import lecturequiz.teacherclient.MainMenu;
import lecturequiz.teacherclient.service.ConnectionManager;

/**
 * A Plain game implementation with four possbile answers to each question
 * and a time limit on each question. All answers are treated equal and statistics
 * are shown after each question and summary after the quiz is done
 * @author knutant
 */
public class PlainGame extends AbstractGameMode implements KeyListener {
    protected Timer questionTimer;
    protected Timer gameStatusTimer;

    public PlainGame(JComponent parent) {
        super(parent);
    }

    public void createRenderer() {
        renderer = new PlainGameRenderer(parent.getWidth(), parent.getHeight());
        parent.add(renderer,BorderLayout.CENTER);

        renderer.addKeyListener(this);
    }

    /**
     * Called when a quiz is started from the UI
     * @param quizId the ID of the quiz started
     * @param quizCode the quiz code of the quiz started
     */
    public void startQuiz(int quizId, String quizCode) {
        // Get the game status to show on the welcome screen
        Holder<GameStatus> gs = new Holder<GameStatus>();

        int ret = ConnectionManager.getInstance().getPort().getCurrentGameStatus(gs);
        if(ret != 0) { // Abort and return to main menu
            JOptionPane.showMessageDialog(parent, "Could not get game status. Error code: " + ret + ". Aborting");
            MainMenu.getInstance().setPanel(MainMenu.PANEL.MAIN_PANEL);
            return;
        }

        // Start the timer to callback the game status updates
        ActionListener gameStatusTimeoutTask = new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                Holder<GameStatus> gs = new Holder<GameStatus>();

                int ret = ConnectionManager.getInstance().getPort().getCurrentGameStatus(gs);
                if(ret != 0) { // Abort and return to main menu
                    JOptionPane.showMessageDialog(parent, "Could not get game status. Error code: " + ret + ". Aborting");
                    MainMenu.getInstance().setPanel(MainMenu.PANEL.MAIN_PANEL);
                    return;
                }

                renderer.showWelcomeScreen(gs.value);
            }
        };

        gameStatusTimer = new Timer(4000,gameStatusTimeoutTask);
        gameStatusTimer.setRepeats(true);
        gameStatusTimer.start();

        renderer.showWelcomeScreen(gs.value);
        renderer.requestFocus();
    }

    /**
     * Starts the next question in the quiz
     */
    public void nextQuestion() {
        Holder<QuestionInfo> question = new Holder<QuestionInfo>();

        // Stop the welcome screen updater if this is the first time we show a question
        if(gameStatusTimer != null) {
            gameStatusTimer.stop();
            gameStatusTimer = null;
        }

        int ret = ConnectionManager.getInstance().getPort().startNextQuestion(question);

        switch(ret) {
            case 0:
                break;

            case 5:
                showOverallStatistics();
                return;

            default:
                JOptionPane.showMessageDialog(parent,"Could not start next question. Error code: " + ret);
                return;
        }

        // Start the question timer
        ActionListener questionTimeoutTask = new ActionListener() {
            private int retryAttempts = 0;

            public void actionPerformed(ActionEvent e) {
                // Get the stats from the server
                Holder<List<StatisticsEntry>> stats = new Holder<List<StatisticsEntry>>();
                int ret = ConnectionManager.getInstance().getPort().getQuestionStatistics(stats);

                switch(ret) {
                    case 0: // Success, show statistics
                        renderer.showStatistics(stats.value);
                        return;

                    case 4: // The stats are not ready yet for some reason, try again in a few secs
                        if(retryAttempts++ > 5) {
                            // Abort, we can't do this forever
                            JOptionPane.showMessageDialog(parent,"Could not get statistics for the last question. Aborting");
                            ConnectionManager.getInstance().getPort().endQuiz();
                            MainMenu.getInstance().setPanel(MainMenu.PANEL.MAIN_PANEL);
                            return;
                        }

                        questionTimer.setDelay(2000);
                        questionTimer.start();
                        break;

                    default: // Other error, try again a couple of times
                        if(retryAttempts++ > 3) {
                            // Abort, we can't do this forever
                            JOptionPane.showMessageDialog(parent,"Could not get statistics for the last question. Error code: " + ret + ". Aborting");
                            ConnectionManager.getInstance().getPort().endQuiz();
                            MainMenu.getInstance().setPanel(MainMenu.PANEL.MAIN_PANEL);
                            return;
                        }

                        questionTimer.setDelay(2000);
                        questionTimer.start();
                }
            }
        };

        questionTimer = new Timer(question.value.getTimeleft() * 1000,questionTimeoutTask);
        questionTimer.setRepeats(false);
        questionTimer.start();

        renderer.showQuestion(question.value);
    }

    /**
     * Performs cleanup, removes the canvas from the parent
     */
    public void cleanup() {
        renderer.cleanup();
        parent.remove(renderer);
    }

    /**
     * Retrieves the overall statistics for the quiz, can only be called after the
     * last question.
     */
    protected void showOverallStatistics() {
        Holder<List<StatisticsEntry>> stats = new Holder<List<StatisticsEntry>>();
        int ret = ConnectionManager.getInstance().getPort().getOverallStatistics(stats);

        switch(ret) {
            case 0: // Success
                renderer.showOverallStatistics(stats.value);
                return;

            default:
                JOptionPane.showMessageDialog(parent,"Could not get overall statistics. Error code: " + ret + ". Aborting");
                ConnectionManager.getInstance().getPort().endQuiz();
                MainMenu.getInstance().setPanel(MainMenu.PANEL.MAIN_PANEL);
        }

        
    }

    public void keyTyped(KeyEvent e) {

    }

    public void keyPressed(KeyEvent e) {
        // Start the next question when space is pressed
        if(e.getKeyCode() == e.VK_SPACE)
            nextQuestion();
    }

    public void keyReleased(KeyEvent e) {

    }
}
