/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package lecturequiz.server.gamemodes;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import lecturequiz.server.data.User;
import lecturequiz.server.data.Question;
import lecturequiz.server.data.Quiz;
import lecturequiz.server.data.StatisticsEntry;
import lecturequiz.server.service.GameStatus;
import lecturequiz.server.service.ParameterEntry;

/**
 *
 * @author erlingbo
 */
public abstract class AbstractGameMode {
    protected String id;
    protected String name;
    protected String quizCode;
    protected User owner;
    protected ArrayList<User> players;
    protected Quiz quiz;
    protected boolean canJoinStarted=true;
    protected Question currentQuestion;
    protected int currentQuestionIndex;
    protected Date questionStartTime;
    protected boolean quizHasEnded;
    protected boolean started = false;

    /**
     * Constructor. Sets the quiz and quiz code members and initializes the
     * players array. Also sets the quizHasEnded member to false.
     * @param quiz Quiz the game will use.
     * @param quizCode Quiz code of the game.
     */
    public AbstractGameMode(Quiz quiz,String quizCode){
        this.quiz = quiz;
        this.quizCode = quizCode;
        players = new ArrayList<User>();
        this.quizHasEnded = false;
    }

    /**
     * Returns the game mode ID of the game mode that runs on this game.
     * @return ID of the game mode
     */
    public String getId(){
        return id;
    }

    /**
     * Returns the game mode name of the game mode that runs on this game
     * @return Name of the game mode
     */
    public String getName(){
        return name;
    }
    
    /**
     * Returns the quiz code for this game.
     * @return The quiz code for this game.
     */
    public String getQuizCode() {
        return quizCode;
    }

    /**
     * Tries add a player to the game.
     * @param player The player to add.
     * @param parameters Custom parameters from the client
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - User already in the game<br/>
     * 2 - Join denied<br/>
     */
    public int joinGame(User player, List<ParameterEntry> parameters) {
        if (players.contains(player)){
            return 1;
        }

        if(started && canJoinStarted == false)
            return 2;
        
        players.add(player);

        // The default implementaion ignores the parameters

        return 0;
    }

    /**
     * Check if the game is started.
     * @return True if the game is started, false otherwise.
     */
    public boolean isStarted() {
        return started;
    }

    /**
     * Returns the list of players in this game
     * @return List of all players in the game
     */
    public ArrayList<User> getPlayers(){
        return players;
    }

    /**
     * Returns the number of current players that have joined this game.
     * @return Number of players in the game.
     */
    public int getNumberOfPlayers(){
        return players.size();
    }

    /**
     * Starts the quiz. Also starts the first question. This will mark the quiz
     * as started.
     */
    public void startQuiz(){
        started = true;
        currentQuestionIndex = -1;

        startNextQuestion();
    }

    /**
     * Returns the current question in the quiz. Should only return a question
     * that is actually running.
     * @return The current question or null if there are no current question.
     */
    public Question getCurrentQuestion() {
        // if the timeout of the question has been reached, we're at the statsitics
        // phase, so return null
        if(currentQuestion == null || timeoutReached())
            return null;

        return currentQuestion;
    }

    /**
     * Starts the next question in the quiz
     * @return True for success, false if no more questions are vailable
     */
    public abstract boolean startNextQuestion();

     /**
     * Returns the combined statistics for all users of this game
     * @return A list of statistics entries, or null if statistics cannot be
     * returned a this point
     */
    public abstract ArrayList<StatisticsEntry> getQuestionStatistics();

    /**
     * Returns the statistics for the last question for the requested user. 
     * @param user User to get statistics for, pass null for general statistics.
     * @return A list of statistics entries, or null if statistics cannot be
     * returned a this point
     */
    public abstract ArrayList<StatisticsEntry> getQuestionStatistics(User user);

    /**
     * Returns the overall statistics for the entire game for all users.
     * @return A list of statistics entries or null if the game is not over.
     */
    public abstract ArrayList<StatisticsEntry> getOverallStatistics();

    /**
     * Returns the overall statistics for the entire game for a specified user.
     * @param user The user to get statistics for
     * @return A list of statistics entries or null if the game is not over.
     */
    public abstract ArrayList<StatisticsEntry> getOverallStatistics(User user);

    /**
     * Checks if the current question running has reached the time limit set by
     * the question.
     * @return True if the timelimit has been reached, false otherwise. If the first \
     * question has not been started, it also returns true.
     */
    public boolean hasQuestionTimedOut() {
        if(currentQuestion == null)
            return true;

        return timeoutReached();
    }

    /**
     * Tells if a client can join this game mode if the first question has already started.
     * @return True if a client can join, false if otherwise.
     */
    public boolean canJoinStarted(){
        return canJoinStarted;
    }

     /**
     * Receives an answer from a player and registers it. Does not check if the answer is correct
     *
     * @param answerID
     * @param player
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Invalid answer<br/>
     * 2 - Player has already answered<br/>
     * 3 - Question timeout is reached
     */
    public abstract int receiveAnswer(int answerID, User player);

    /**
     * Set the owner of this game
     * @param user the user to set as owner
     */
    public void setOwner(User user) {
        owner = user;
    }

    /**
     * Return the owner of this game
     * @return the game owner
     */
    public User getOwner() {
        return owner;
    }

    /**
     * Remove a user from the game
     * @param user user to remove
     */
    public void removeUser(User user) {
        players.remove(user);
    }

    /**
     * Tells if the quiz has ended and the last question has been answered
     * @return True if the quiz has ended, false if not
     */
    public boolean hasQuizEnded() {
        return quizHasEnded;
    }

    /**
     * Get the current status of this game, such as players in game, number of
     * teams, what question we are on etc.
     * @return A GameStatus object with the status of this game
     */
    public GameStatus getGameStatus() {
        GameStatus gs = new GameStatus();
        gs.playerCount = new ArrayList<Integer>();
        gs.playerCount.add(players.size());
        gs.quizCode = quizCode;
        gs.currentQuestion = currentQuestionIndex + 1;
        gs.questionCount = quiz.getNumberOfQuestions();

        return gs;
    }

    /**
     * Sets the custom parameters for this game mode. Is usually called on game creation.
     * The default implenetation ignores these parameters.
     * @param parameters The custom parameters from the teacher client who created the game.
     * @return True if the parameters are valid and accepted, false otherwise.
     */
    public boolean setParameters(List<ParameterEntry> parameters) {
        // The default implementation ignores custom parameters
        return true;
    }

    /**
    * Returns the time left before the question should change. Will return 0 if
    * there is no current question
    * @return the time left in seconds
    */
    public int getTimeLeft() {
        if(currentQuestion == null)
            return 0;

        int diff = (int)((new Date().getTime() - questionStartTime.getTime()) / 1000);
        return currentQuestion.getTimeout() - diff;
    }

    /**
     * Simple function to determine if the time limit of a question has been reached
     * @return True if the time limit has been reached, false otherwise. Will return
     * true if there is no current question.
     */
    protected boolean timeoutReached() {
        return (getTimeLeft() <= 0);
    }

}
