/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package lecturequiz.server.gamemodes;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;
import lecturequiz.server.data.User;
import lecturequiz.server.data.Quiz;
import lecturequiz.server.data.StatisticsEntry;

/**
 *
 * @author erlingbo
 */
public class PlainGame extends AbstractGameMode {
    protected HashMap<User, Integer> currentAnswers;
    protected ArrayList<HashMap<User, Integer>> statistics;

    /**
     * Constructor. Calls the AbstractGameMode constructor and sets the id and
     * name of this game mode.
     * @param quiz Quiz the game will use.
     * @param quizCode Quiz code of the game.
     */
    public PlainGame(Quiz quiz, String quizCode){
        super(quiz,quizCode);
        id = "PlainGame";
        name = "Plain Game";
    }


    /**
     * Starts the quiz. Also starts the first question. This will mark the quiz
     * as started.
     */
    @Override
    public void startQuiz() {

        currentAnswers = null;
        statistics = new ArrayList<HashMap<User, Integer>>();

        super.startQuiz();
    }

    /**
     * Starts the next question in the quiz
     * @return True for success, false if no more questions are vailable
     */
    public boolean startNextQuestion(){
        if (currentQuestionIndex < (quiz.getNumberOfQuestions() - 1)){
            currentQuestionIndex++;
            if(currentAnswers != null)
                statistics.add(currentAnswers);
            currentAnswers = new HashMap<User, Integer>();
            currentQuestion = quiz.getQuestions().get(currentQuestionIndex);
            questionStartTime = new Date();
            return true;
        }
        else {
            //no more questions available
            quizHasEnded = true;
            return false;
        }
    }

    @Override
    public synchronized int receiveAnswer(int answerID, User player) {
        if(currentQuestion == null)
            return 3;

        if (!currentQuestion.hasAnswer(answerID)){
            return 1;

        }
        else if (currentAnswers.containsKey(player)){
            return 2;
        }
        else if (timeoutReached()){
            return 3;
        }
        else{
            currentAnswers.put(player, answerID);
            return 0;
        }   
    }

    @Override
    public void removeUser(User user) {
        // remove any answers by this user
        if(currentAnswers != null)
            currentAnswers.remove(user);
        
        super.removeUser(user);
    }

    /**
     * Returns the statistics for the last question for the all the users in the quiz.
     * This implementation returns the correct answer id and the percentage of
     * users who chose the different answer options. The answer is stored in the
     * "currentAnswer" key and the percentage of users who chose the different anwers
     * have the key of the answer ID they represent.
     * @return A list of statistics entries, or null if statistics cannot be
     * returned a this point
     */
    @Override
    public ArrayList<StatisticsEntry> getQuestionStatistics()
    {
        // Check if the question answer time has ran out and fail if it hasn't
        if(!timeoutReached())
            return null;

        ArrayList<StatisticsEntry> list = new ArrayList<StatisticsEntry>();

        // The correct answer should be part of both statistics
        StatisticsEntry e = new StatisticsEntry();
        e.key = "correctAnswer";
        e.value = String.valueOf(currentQuestion.getCorrectAnswer());
        list.add(e);

        /*
         * TODO: We could populate this list while people are answering \
         * to speed up things
         */
        HashMap<Integer,Integer> answers = new HashMap<Integer,Integer>();
        for(int i = 0; i < currentQuestion.getAnswers().size(); i++) {
            answers.put(currentQuestion.getAnswers().get(i).id, 0);
        }

        // Count all the answers for each answer-option
        for(Iterator<Integer> i = currentAnswers.values().iterator(); i.hasNext(); ) {
            Integer val = i.next();
            answers.put(val,answers.get(val)+1);
        }

        // Add all answers as entries
        for(Iterator<Integer> i = answers.keySet().iterator(); i.hasNext(); ) {
            e = new StatisticsEntry();
            Integer key = i.next();
            e.key = key.toString();
            e.value = answers.get(key).toString();
            list.add(e);
        }

        return list;
    }

    /**
     * Returns the statistics for the last question for the requested user.
     * This implementation only returns the correct answer id stored in the
     * "correctAnswer" key.
     * @param user User to get statistics for, pass null for general statistics.
     * @return A list of statistics entries, or null if statistics cannot be
     * returned a this point
     */
    @Override
    public ArrayList<StatisticsEntry> getQuestionStatistics(User user)
    {
        // Check if the question answer time has ran out and fail if it hasn't
        if(!timeoutReached())
            return null;

        if(user == null)
            return null;
        
        ArrayList<StatisticsEntry> list = new ArrayList<StatisticsEntry>();
        
        // The correct answer should be part of both statistics
        StatisticsEntry e = new StatisticsEntry();
        e.key = "correctAnswer";
        e.value = String.valueOf(currentQuestion.getCorrectAnswer());
        list.add(e);
        
        return list;
    }

    /**
     * Returns the overall statistics for the entire game.
     * In this game mode we return the percentage of answers that was correct
     * and which users who answered the most correct questions<br/>
     * <b>Fields:</b><br/>
     * "correct" - The percentage of correct answers as a float in string form<br/>
     * "questionCount" - The number of questions in this quiz<br/>
     * "mostCorrectUser" - The user who had the most correct answers<br/>
     * "mostCorrectCount" - The number of correct answers this user had<br/>
     * @return A list of statistics entries or null if the game is not over.
     */
    @Override
    public ArrayList<StatisticsEntry> getOverallStatistics()
    {
        // Can only get stats if the answer time is over
        if(!timeoutReached())
            return null;
        
        // And only if the previous question was the last in the quiz
        if(currentQuestionIndex != (quiz.getNumberOfQuestions() -1))
            return null;

        // TODO: Populate a list here, or return a pregenerated one
        ArrayList<StatisticsEntry> stats = new ArrayList<StatisticsEntry>();

        HashMap<User,Integer> correctAnswers = new HashMap<User, Integer>();
        int correctCount = 0;
        int totalAnswers = 0;

        // Sum up all the answers for each user
        for(int i = 0; i < statistics.size(); i++) {
            for(Iterator<Entry<User, Integer>> j = statistics.get(i).entrySet().iterator(); j.hasNext();) {
                Entry<User, Integer> e = j.next();

                if(e.getValue() == quiz.getQuestions().get(i).getCorrectAnswer()) {
                    correctCount++;
                    if(correctAnswers.containsKey(e.getKey()))
                        correctAnswers.put(e.getKey(), correctAnswers.get(e.getKey()) + 1);
                    else
                        correctAnswers.put(e.getKey(), 1);
                }

                totalAnswers++;
            }
        }

        // Also count up the answers from the last question
        for(Iterator<Entry<User, Integer>> j = currentAnswers.entrySet().iterator(); j.hasNext();) {
            Entry<User, Integer> e = j.next();

            if(e.getValue() == currentQuestion.getCorrectAnswer()) {
                correctCount++;
                if(correctAnswers.containsKey(e.getKey()))
                    correctAnswers.put(e.getKey(), correctAnswers.get(e.getKey()) + 1);
                else
                    correctAnswers.put(e.getKey(), 1);
            }

            totalAnswers++;
        }

        // Find the user with highest score, if there's multiple pick the first one
        User mostCorrectUser = null;
        int mostCorrectCount = 0;
        for(Iterator<Entry<User, Integer>> i = correctAnswers.entrySet().iterator(); i.hasNext();) {
            Entry<User, Integer> e = i.next();

            if(e.getValue() > mostCorrectCount) {
                mostCorrectCount = e.getValue();
                mostCorrectUser = e.getKey();
            }
        }

        StatisticsEntry e = new StatisticsEntry();
        e.key = "correct";
        e.value = Float.toString((float)correctCount / (float)totalAnswers);
        stats.add(e);

        e = new StatisticsEntry();
        e.key = "questionCount";
        e.value = Integer.toString(quiz.getNumberOfQuestions());
        stats.add(e);

        e = new StatisticsEntry();
        e.key = "mostCorrectUser";

        // If everyone sucked and nobody had a correct answer, we will end up here
        if(mostCorrectUser == null)
            e.value = null;
        else
            e.value = mostCorrectUser.getNickname();

        stats.add(e);

        e = new StatisticsEntry();
        e.key = "mostCorrectCount";
        e.value = Integer.toString(mostCorrectCount);
        stats.add(e);

        return stats;
    }

    /**
     * Returns the overall statistics for the entire game for a specified user.
     * In this game mode we only return the percentage of answers that were correct<br/>
     * <b>Fields:</b><br/>
     * "correct" - The percentage of correct answers as a float in string form
     * @param user The user to get statistics for
     * @return A list of statistics entries or null if the game is not over.
     */
    public ArrayList<StatisticsEntry> getOverallStatistics(User user) {

        // Can only get stats if the answer time is over
        if(!timeoutReached())
            return null;

        // And only if the previous question was the last in the quiz
        if(currentQuestionIndex != (quiz.getNumberOfQuestions() -1))
            return null;

        ArrayList<StatisticsEntry> stats = new ArrayList<StatisticsEntry>();

        int correctAnswers = 0;

        // loop through all previous answers for this user
        for(int i = 0; i < statistics.size(); i++) {
            Integer answer = statistics.get(i).get(user);

            if(answer != null)
                if(answer == quiz.getQuestions().get(i).getCorrectAnswer())
                    correctAnswers++;
        }

        // Make sure we get the last question counted aswell
        Integer answer = currentAnswers.get(user);
        if(answer != null)
            if(answer == currentQuestion.getCorrectAnswer())
                correctAnswers++;

        StatisticsEntry e = new StatisticsEntry();
        e.key = "correct";
        e.value = Float.toString((float)correctAnswers / quiz.getNumberOfQuestions());
        stats.add(e);

        return stats;
    }
}
