/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package lecturequiz.server.main;

import java.util.ArrayList;
import java.util.List;
import lecturequiz.server.data.Quiz;
import lecturequiz.server.data.User;
import lecturequiz.server.gamemodes.AbstractGameMode;
import lecturequiz.server.gamemodes.IGameModeFactory;
import lecturequiz.server.gamemodes.PlainGameFactory;
import lecturequiz.server.service.GameModeInfo;
import lecturequiz.server.service.ParameterEntry;

/**
 *
 * @author avenger
 */
public class GameManager {
    // Singleton instance of this class
    private static GameManager _instance = null;

    // Supported game types
    private ArrayList<IGameModeFactory> supportedModes;
    
    // Currently ongoing games
    private ArrayList<AbstractGameMode> games;


    /**
     * Private constructor
     */
    private GameManager() {
        games = new ArrayList<AbstractGameMode>();

        supportedModes = new ArrayList<IGameModeFactory>();
        supportedModes.add(new PlainGameFactory());
    }

    /**
     * Get the GameManager instance, or create a new one if it doesn't exist
     * @return The GameManager instance
     */
    public static synchronized GameManager getInstance() {
        if(_instance == null)
            _instance = new GameManager();

        return _instance;
    }

    /**
     * Check if the supplied game mode is supported by the game server.
     * @param gameMode The game mode to check.
     * @return True if  the mode is supported, false otherwise.
     */
    public boolean isGameModeSupported(String gameMode) {
        for(int i = 0; i < supportedModes.size(); i++)
            if(supportedModes.get(i).getGameModeID().equals(gameMode))
                return true;

        return false;
    }

    /**
     * Creates a new game instance with the supplied game mode, quiz and quiz code.
     * @param gameMode The game mode of this game
     * @param quizId The ID of the quiz to start
     * @param quizCode The quiz code needed to join this game
     * @param parameters Custom parameters sent to the game mode on creation. Can be null.
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Quiz not found<br/>
     * 2 - Unsupported game mode<br/>
     * 3 - Invalid parameters<br/>
     */
    public int newQuiz(String gameMode, int quizId, String quizCode, User owner,
            List<ParameterEntry> parameters) {

        if(!isGameModeSupported(gameMode)) {
            return 1;
        }

        // Get the Quiz from the database
        Quiz quiz = DBManager.getInstance().getQuiz(quizId);

        if(quiz == null)
            return 2;
        
        /**
         * FIXME: Supported game modes are looped twice, maybe fix this?
         */
        
        AbstractGameMode game = null;
        for(int i = 0; i < supportedModes.size(); i++)
            if(supportedModes.get(i).getGameModeID().equals(gameMode))
                game = supportedModes.get(i).createInstance(quiz,quizCode);

        if(game == null)
            return 1;

        if(!game.setParameters(parameters))
            return 3;

        // Associate this user with the newly created game
        owner.setCurrentGame(game);
        game.setOwner(owner);

        // Add to the list of ongoing games
        games.add(game);

        return 0;
    }

    /**
     * Returns a game instance corresponding to the given quiz code
     * @param quizCode the quiz code of the game to return
     * @return game instance with the given quiz code
     */
    public AbstractGameMode getGame(String quizCode) {
        for(int i = 0; i < games.size(); i++)
            if(games.get(i).getQuizCode().equals(quizCode))
                return games.get(i);

        return null;
    }

    /**
     * Returns a list of the supported game modes
     * @return the list of supported game modes
     */
    public ArrayList<GameModeInfo> getSupportedGameModes() {
        ArrayList<GameModeInfo> gameModes = new ArrayList<GameModeInfo>();

        for(int i = 0; i < supportedModes.size(); i++) {
            GameModeInfo gi = new GameModeInfo();
            gi.id = supportedModes.get(i).getGameModeID();
            gi.gameName = supportedModes.get(i).getGameModeName();
            gameModes.add(gi);
        }

        return gameModes;
    }

    public void endGame(AbstractGameMode game) {
        // Kick out all the players of this game
        ArrayList<User> players = game.getPlayers();

        for(int i = 0; i < players.size(); i ++) {
            UserManager.getInstance().removeUser(players.get(i));
        }

        game.setOwner(null);

        games.remove(game);
    }
}
