/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package lecturequiz.server.service;

import java.util.ArrayList;
import java.util.List;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.xml.ws.Holder;
import javax.xml.ws.WebServiceContext;
import javax.annotation.Resource;
import javax.jws.WebParam.Mode;
import javax.servlet.http.HttpSession;
import javax.xml.ws.handler.MessageContext;
import lecturequiz.server.data.Quiz;
import lecturequiz.server.data.StatisticsEntry;
import lecturequiz.server.data.User;
import lecturequiz.server.gamemodes.AbstractGameMode;
import lecturequiz.server.main.DBManager;
import lecturequiz.server.main.GameManager;
import lecturequiz.server.main.UserManager;

/**
 *
 * @author avenger
 */
@WebService()
public class LectureQuizService {

    @Resource
    private WebServiceContext wsContext;

    private HttpSession getSession() {
        MessageContext mc = wsContext.getMessageContext();
        return ((javax.servlet.http.HttpServletRequest)mc.get(MessageContext.SERVLET_REQUEST)).getSession();
    }

    /**
     * Authenticate and create a session.
     * @param username The username to be used in the game.
     * @param password A password to authenticate with.
     * @param auth Set to true if the user should try to authenticate with the password,
     * otherwise just check for username availability and log in as a normal student.
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Authentication failed.<br/>
     * 2 - Username already in use.<br/>
     * 3 - Invalid parameters<br/>
     */
    @WebMethod(operationName = "authenticate")
    public int authenticate(@WebParam(name = "username")
            String username, @WebParam(name = "password")
            String password, @WebParam(name = "auth")
            boolean auth) {

        User user;
        UserManager um = UserManager.getInstance();

        // Auth against DB only if it's requested
        if(auth) {
            if(username == null || username.isEmpty() || password == null)
                return 3;

            user = UserManager.getInstance().authenticateUser(username, password);

            if(user == null)
                return 1;
        } else {
            if(username == null || username.isEmpty())
                return 3;

            if(!um.isUsernameAvailable(username))
                return 2;

            user = um.addUser(username,User.STUDENT);
        }

        HttpSession session = getSession();
        // If the user has already authenticated, remove the previous user, so it
        // won't be lost in the system
        if(session.getAttribute("User") != null) {
            UserManager.getInstance().removeUser((User)session.getAttribute("User"));
        }

        // Set the user object in the session so we know that the user has logged in
        session.setAttribute("User", user);
        return 0;
    }

    /**
     * Get a list of the available quizzes
     * @param quizzes <b>OUT:</b> An ArrayList of QuizInfo objects populated by 
     * this function in the form of a Hoder<T> type.
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - Access denied<br/>
     * 4 - Error occurred on the server<br/>
     */
    @WebMethod(operationName = "getAvailableQuizzes")
    public int getAvailableQuizzes(@WebParam(name = "quizzes", mode = Mode.OUT)
            Holder<List<QuizInfo> > quizzes) {

        HttpSession session = getSession();
        if(session == null)
            return 1;

        if(session.getAttribute("User") == null)
            return 2;

        // Check for permissions
        int role = ((User)session.getAttribute("User")).getRole();
        if(role != User.TEACHER && role != User.ADMIN)
            return 3;

        ArrayList<QuizInfo> list = DBManager.getInstance().getQuizList();

        if(list == null)
            return 4;

        quizzes.value = list;

        return 0;
    }

    /**
     * Get a list of supported game modes on the server
     * @param gameModes
     * @return Error codes:<br/>
     * 0 - Success
     */
    @WebMethod(operationName = "getAvailableGameModes")
    public int getAvailableGameModes(@WebParam(name = "gameModes", mode = Mode.OUT)
            Holder<List<GameModeInfo>> gameModes) {

        gameModes.value = GameManager.getInstance().getSupportedGameModes();

        return 0;
    }

    /**
     * Start a new quiz on the server. Does not start the first question.
     * @param gameMode ID of the game mode to use with this quiz
     * @param quizId ID of the quiz to start
     * @param quizCode The code clients should supplement to join this quiz
     * @param parameters Custom parameters sent to the game mode on creation. Can be null.
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - Access denied<br/>
     * 4 - Quiz code in use<br/>
     * 5 - Unsupported game mode<br/>
     * 6 - Quiz not found<br/>
     * 7 - Invalid parameters<br/>
     */
    @WebMethod(operationName = "newQuiz")
    public int newQuiz(@WebParam(name = "gameMode")
            String gameMode, @WebParam(name = "quizId")
            int quizId, @WebParam(name = "quizCode")
            String quizCode, @WebParam(name = "parameters")
            List<ParameterEntry> parameters) {

        HttpSession session = getSession();
        if(session == null)
            return 1;
        if(session.getAttribute("User") == null)
            return 2;

        // Check for permissions
        User user = (User)session.getAttribute("User");
        if(user.getRole() != User.TEACHER && user.getRole() != User.ADMIN)
            return 3;

        // Make sure the quiz code isn't in use by another game
        if(GameManager.getInstance().getGame(quizCode) != null)
            return 4;

        // Start the new game and check for errors
        switch(GameManager.getInstance().newQuiz(gameMode, quizId, quizCode, user, parameters)) {
            case 1:
                return 5;
            case 2:
                return 6;
            case 3:
                return 7;
        }

        return 0;
    }

    /**
     * Starts the next question. This function is only available to authenticated users.
     * @param info <b>OUT:</b> The question information of the question to be displayed is placed in this object.
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - Access denied<br/>
     * 4 - No active quiz<br/>
     * 5 - No more questions in this quiz<br/>
     * 6 - Another question is currently active<br/>
     */
    @WebMethod(operationName = "startNextQuestion")
    public int startNextQuestion(@WebParam(name = "info", mode = Mode.OUT)
            Holder<QuestionInfo> info) {

        HttpSession session = getSession();
        if(session == null)
            return 1;

        if(session.getAttribute("User") == null)
            return 2;

        // Check for permissions
        User user = (User)session.getAttribute("User");
        if(user.getRole() != User.TEACHER && user.getRole() != User.ADMIN)
            return 3;

        AbstractGameMode game = user.getCurrentGame();
        if(game == null)
            return 4;

        // Start the quiz if this is needed
        if(!game.isStarted())
            game.startQuiz();
        else {
            if(!game.hasQuestionTimedOut())
                return 6;

            if(!game.startNextQuestion()) {
                return 5;
            }
        }

        info.value = game.getCurrentQuestion().getQuestionInfo();
        info.value.timeleft = game.getTimeLeft();

        return 0;
    }

    /**
     * Returns the current question in the quiz
     * @param info <b>OUT:</b> The question information is placed in this object.
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - No active quiz<br/>
     * 4 - Cannot get question at this time<br/>
     * 5 - No more questions in the quiz<br/>
     */
    @WebMethod(operationName = "getCurrentQuestion")
    public int getCurrentQuestion(@WebParam(name = "info", mode = Mode.OUT)
            Holder<QuestionInfo> info) {

        HttpSession session = getSession();
        if(session == null)
            return 1;

        User user = (User)session.getAttribute("User");
        if(user == null)
            return 2;
        
        if(user.getCurrentGame() == null)
            return 3;

        if(user.getCurrentGame().hasQuizEnded())
            return 5;

        if(!user.getCurrentGame().isStarted() ||
            user.getCurrentGame().getCurrentQuestion() == null)
            return 4;

        info.value = user.getCurrentGame().getCurrentQuestion().getQuestionInfo();
        
        // make sure the correct time left value is set
        info.value.timeleft = user.getCurrentGame().getTimeLeft();

        return 0;
    }

    /**
     * Returns the running game mode of the supplied quiz code
     * @param quizCode the code of the quiz to check
     * @param info the game mode info is stored in this object
     * @return Error codes:<br/>
     * 0 - Success
     * 1 - No such quiz
     */
    @WebMethod(operationName = "getGameModeInfo")
    public int getGameModeInfo(@WebParam(name = "quizCode") String quizCode,
            @WebParam(name = "info", mode = Mode.OUT) Holder<GameModeInfo> info) {

        AbstractGameMode game = GameManager.getInstance().getGame(quizCode);
        if(game == null)
            return 1;

        GameModeInfo gmi = new GameModeInfo();


        gmi.id = game.getId();
        gmi.gameName = game.getName();

        info.value = gmi;

        return 0;
    }

    /**
     * Attempt to join a quiz with the supplied quiz code
     * @param quizCode A code that identifies the quiz you want to join
     * @param parameters Custom parameters sent to the game mode on join. Can be null.
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - No such quiz<br/>
     * 4 - Join denied<br/>
     */
    @WebMethod(operationName = "joinQuiz")
    public int joinQuiz(@WebParam(name = "quizCode")
            String quizCode, @WebParam(name = "parameters")
            List<ParameterEntry> parameters) {

        HttpSession session = getSession();
        if(session == null)
            return 1;

        User user = (User)session.getAttribute("User");
        if(user == null)
            return 2;

        if(user.getRole() != User.STUDENT)
            return 4;

        AbstractGameMode game = GameManager.getInstance().getGame(quizCode);
        if(game == null)
            return 3;

        int jgRetVal = game.joinGame(user,parameters);
        if(jgRetVal == 1)
            return 5;
        if(jgRetVal != 0)
            return 4;
        


        user.setCurrentGame(game);
        
        return 0;
    }

    /**
     * Submits an answer to the current quiz.
     * @param answerId The ID of the answer.
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - Not a part of any quiz<br/>
     * 4 - Not a valid answer<br/>
     * 5 - User has already answered<br/>
     * 6 - Quiz does not accept answers at this time<br/>
     */
    @WebMethod(operationName = "submitAnswer")
    public int submitAnswer(@WebParam(name = "answerId") int answerId) {

        // Make sure the user is logged in and part of a game
        HttpSession session = getSession();
        if(session == null)
            return 1;

        User user = (User)session.getAttribute("User");
        if(user == null)
            return 2;

        if(user.getCurrentGame() == null)
            return 3;

        // Try to answer the question
        switch(user.getCurrentGame().receiveAnswer(answerId, user)) {
            case 1:
                return 4;
            case 2:
                return 5;
            case 3:
                return 6;
        }

        return 0;
    }

    /**
     * Ends the quiz the calling user is running
     * @return Error codes:<br/>
     * 0 - Success
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - Access denied<br/>
     * 4 - No active quiz<br/>
     */
    @WebMethod(operationName = "endQuiz")
    public int endQuiz() {

        HttpSession session = getSession();
        if(session == null)
            return 1;

        if(session.getAttribute("User") == null)
            return 2;

        // Check for permissions
        User user = (User)session.getAttribute("User");
        if(user.getRole() != User.TEACHER && user.getRole() != User.ADMIN)
            return 3;

        AbstractGameMode game = user.getCurrentGame();
        if(game == null)
            return 4;

        GameManager.getInstance().endGame(game);

        return 0;
    }

    /**
     * Get statistics for the last question answered
     * @param stats the statistics are places in this holder
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - No active quiz<br/>
     * 4 - Cannot get statistics at this time<br/>
     */
    @WebMethod(operationName = "getQuestionStatistics")
    public int getQuestionStatistics(@WebParam(name = "stats", mode = Mode.OUT)
            Holder<List<StatisticsEntry>> stats) {

        HttpSession session = getSession();
        if(session == null)
            return 1;

        User user = (User)session.getAttribute("User");
        if(user == null)
            return 2;

        AbstractGameMode game = user.getCurrentGame();
        if(game == null)
            return 3;
        
        // A teacher / admin will get different statistics from a student client
        if(user.getRole() == User.TEACHER || user.getRole() == User.ADMIN) {
            stats.value = game.getQuestionStatistics();
        } else {
            stats.value = game.getQuestionStatistics(user);
        }

        if(stats.value == null)
            return 4;
        
        return 0;
    }
    /**
     * Get the overall statistics from the entire game
     * @param stats The statistics are placed in this holder
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - No active quiz<br/>
     * 4 - Cannot get statistics at this time<br/>
     */
    public int getOverallStatistics(@WebParam(name = "stats", mode = Mode.OUT)
            Holder<List<StatisticsEntry>> stats) {

        HttpSession session = getSession();
        if(session == null)
            return 1;

        User user = (User)session.getAttribute("User");
        if(user == null)
            return 2;

        AbstractGameMode game = user.getCurrentGame();
        if(game == null)
            return 3;

        ArrayList<StatisticsEntry> list;
        if(user.getRole() == User.ADMIN || user.getRole() == User.TEACHER)
            list = game.getOverallStatistics();
        else
            list = game.getOverallStatistics(user);

        if(list == null)
            return 4;

        stats.value = list;

        return 0;
    }

     /**
     * Get the status of the current game
     * Can only be called by teacher clients.
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - Access denied<br/>
     * 4 - No active quiz<br/>
     */
    @WebMethod(operationName = "getCurrentGameStatus")
    public int getCurrentGameStatus(@WebParam(name = "gameStatus", mode = Mode.OUT)
            Holder<GameStatus> gameStatus) {

        HttpSession session = getSession();
        if(session == null)
            return 1;

        if(session.getAttribute("User") == null)
            return 2;

        // Check for permissions
        User user = (User)session.getAttribute("User");
        if(user.getRole() != User.TEACHER && user.getRole() != User.ADMIN)
            return 3;

        AbstractGameMode game = user.getCurrentGame();
        if(game == null)
            return 4;

        gameStatus.value = game.getGameStatus();

        return 0;
    }

    /**
     * Retrieve a full quiz with all questions and answers, used for editing
     * Can only be called by teachers and admins
     * @param id ID of the quiz to retrieve
     * @param quizInfo The quiz information will be placed in this object
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - Access denied<br/>
     * 4 - No such quiz<br/>
     */
    @WebMethod(operationName = "getQuiz")
    public int getQuiz(@WebParam(name = "id") int id,
            @WebParam(name = "quizInfo", mode = Mode.OUT) Holder<FullQuizInfo> quizInfo) {
        
        HttpSession session = getSession();
        if(session == null)
            return 1;

        if(session.getAttribute("User") == null)
            return 2;

        // Check for permissions
        User user = (User)session.getAttribute("User");
        if(user.getRole() != User.TEACHER && user.getRole() != User.ADMIN)
            return 3;

        Quiz q = DBManager.getInstance().getQuiz(id);
        if(q == null)
            return 4;

        quizInfo.value = q.toFullQuizInfo();

        return 0;
    }

    /**
     * Returns a list of all questions stored in the database. Used in conjunction
     * with quiz editing.
     * @param info A list of FullQuestionInfo objects. The questions are stored in
     * these objects. Be aware that only the correct answer is added to the list
     * of answers in this section to increase performance.
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - Access denied<br/>
     * 4 - Could not get list from DB<br/>
     */
    @WebMethod(operationName = "getQuestionList")
    public int getQuestionList(@WebParam(name = "info", mode = Mode.OUT)
            Holder<List<FullQuestionInfo>> info) {

        HttpSession session = getSession();
        if(session == null)
            return 1;

        if(session.getAttribute("User") == null)
            return 2;

        // Check for permissions
        User user = (User)session.getAttribute("User");
        if(user.getRole() != User.TEACHER && user.getRole() != User.ADMIN)
            return 3;

        info.value = DBManager.getInstance().getQuestionList();
        if(info.value == null)
            return 4;

        return 0;

    }

    /**
     *
     * @param id
     * @param info
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - Access denied<br/>
     * 4 - No such question<br/>
     */
    @WebMethod(operationName = "getQuestion")
    public int getQuestion(@WebParam(name = "id") int id,
            @WebParam(name = "info",mode = Mode.OUT) Holder<FullQuestionInfo> info) {

        HttpSession session = getSession();
        if(session == null)
            return 1;

        if(session.getAttribute("User") == null)
            return 2;

        // Check for permissions
        User user = (User)session.getAttribute("User");
        if(user.getRole() != User.TEACHER && user.getRole() != User.ADMIN)
            return 3;

        info.value = DBManager.getInstance().getQuestion(id);
        if(info.value == null)
            return 4;
        
        return 0;
    }

    /**
     * Saves a quiz
     * @param quizInfo the information about the quiz to save. If the
     * FullQuizInfo.id is 0 a new quiz is created. A user with the role of teacher
     * can only save his own quiz. Any question with the id of 0 is
     * also created, answers should then be listed with ids of 0-n and correctAnswer
     * should point to these ids.
     * @return Error codes:<br/>
     * 0 - Success<br/>
     * 1 - Could not find a valid session<br/>
     * 2 - Not logged in<br/>
     * 3 - Access denied<br/>
     * 4 - No such quiz<br/>
     * 5 - Invalid data<br/>
     * 6 - Unknown error<br/>
     */
    @WebMethod(operationName = "saveQuiz")
    public int saveQuiz(@WebParam(name = "quizInfo") FullQuizInfo quizInfo) {

        HttpSession session = getSession();
        if(session == null)
            return 1;

        if(session.getAttribute("User") == null)
            return 2;

        // Check for permissions
        User user = (User)session.getAttribute("User");
        if(user.getRole() != User.TEACHER && user.getRole() != User.ADMIN)
            return 3;

        switch(DBManager.getInstance().saveQuiz(quizInfo, user)) {
            case 0:
                return 0;

            case 1:
                return 4;

            case 2:
                return 5;

            case 3:
                return 3;

            case 4:
            default:
                return 6;
        }
    }

    /**
     * Get the version of lecture quiz this server is running
     * @param version The version number is put into this variable
     * @return Error codes:<br/>
     * 0 - Success
     */
    @WebMethod(operationName = "getServiceVersion")
    public int getServiceVersion(@WebParam(name = "version", mode = Mode.OUT) Holder<String> version) {
        version.value = "2.0.0";

        return 0;
    }

}
